#!/usr/local/bin/perl
use strict;
use warnings;

open my $fh, '<', 'oscars.txt' or die "Cannot open input: $!\n";
my %award_count;
while ( <$fh> )
{
  chomp;
  my ($for, $rest) = /([^:]+): (.*)/ or next;
  if ( $for =~ /\AAct/ )
  {
    my ($who) = ( $rest =~ /(.*) as / ) or next;
    $award_count{$who}++;
  }
  else
  {
    my ($list) = ( $rest =~ /(.*\S)\s+for / ) or next;
    $list =~ s/(Co-)?Producers?;?//;
    $list =~ s/.* by //;
    my @people = split /, | and | & /, $list;
    $award_count{$_}++ for @people;
  }
}

my $top_count;
for my $who ( sort { $award_count{$b} <=> $award_count{$a} } keys %award_count )
{
  my $count = $award_count{$who};
  if ( $top_count )
  {
    last if $count < $top_count;
  }
  else
  {
    $top_count = $count;
  }
  print "$who has $count awards\n";
}

__END__

=head1 NAME

oscar_stats.pl - Glean some information from a list of Academy Award winners.

=head1 SYNOPSIS

  oscar_stats.pl

=head1 DESCRIPTION

This version of the C<oscar_stats.pl> program scans the input for
names and counts their occurrences to report on who appears the most
times, regardless of category.  If the most frequent appearance is a
tie, report all the people with that many appearances.

=head1 BUGS

None so far.  The input data is not strictly formatted and any other
version of it for earlier or later years might employ formatting that
breaks the assumptions of this code.

=head1 AUTHOR

Peter Scott, C<< <Peter@PSDT.com> >>

=head1 LICENSE

Program code copyright Addison-Wesley, 2012, as part of the I<Beyond
Perl Fundamentals> video course.  Input data selected and reformatted
from publically available information.

=head1 SEE ALSO

Perl Fundamentals LiveLessons DVD and InformIT online videos.

=cut
