#!/usr/local/bin/perl
use strict;
use warnings;

use List::Util qw(max);

my ($oscars_ref, $movie_ref) = read_oscars( 'oscars.txt' );
my $max = max( map { scalar( keys %$_ ) } values %$movie_ref );
for my $movie ( keys %$movie_ref )
{
  my @categories = keys %{ $movie_ref->{$movie} };
  @categories == $max or next;
  print "$movie won for: ", join( ', ' => @categories), "\n";
}

sub read_oscars
{
  my $input = shift;

  my (%oscars, %movie);

  open my $fh, '<', $input or die "Cannot open $input: $!\n";
  my %award_count;
  my $year;
  while ( <$fh> )
  {
    chomp;
    /\A(\d+)\z/ and $year = $1;
    my ($for, $rest) = /([^:]+): (?:Written for )?(.*)/ or next;
    my @people;
    my $what;
    if ( $for =~ /\AAct/ )
    {
      (my $who, $what) = ( $rest =~ /(.*) as .*? in (.*)/ ) or next;
      @people = $who;
    }
    else
    {
      (my $list, $what) = ( $rest =~ /(.*?\S)\s+for (.*)/ ) or next;
      $list =~ s/, (Co-)?Producers?//g;
      @people = split /, | and | & |;\s+/, $list;
      s/.* by // for @people;
    }
    $movie{$what}{$for} = $oscars{$year}{$for} = \@people;
  }

  return (\%oscars, \%movie);
}
