package Oscars;
use strict;
use warnings;

use Carp;

sub new
{
  my ($class, %arg) = @_;

  my $ref = \%arg;
  bless $ref, $class;
  return $ref;
}

sub populate
{
  my $self = shift;

  my $input = $self->{input} or croak "No input file given";
  my ($oscar_ref, $movie_ref) = read_input( $input );
  $self->{oscars} = $oscar_ref;
  $self->{movies} = $movie_ref;
}

sub winners
{
  my ($self, %criterion) = @_;

  my $year     = $criterion{year} or croak "I need a year";
  my $category = $criterion{category} or croak "I need a category";
  my $year_ref = $self->{oscars}{$year} or croak "I don't have data for $year";
  my $winner_ref = $year_ref->{"\u\L$category"} or croak "I don't have data for $category";
  return @$winner_ref;
}

sub read_input
{
  my $input = shift;

  my (%oscars, %movie);

  open my $fh, '<', $input or die "Cannot open $input: $!\n";
  my %award_count;
  my $year;
  while ( <$fh> )
  {
    chomp;
    /\A(\d+)\z/ and $year = $1;
    my ($for, $rest) = /([^:]+): (?:Written for )?(.*)/ or next;
    my @people;
    my $what;
    if ( $for =~ /\AAct/ )
    {
      (my $who, $what) = ( $rest =~ /(.*) as .*? in (.*)/ ) or next;
      @people = $who;
    }
    else
    {
      (my $list, $what) = ( $rest =~ /(.*?\S)\s+for (.*)/ ) or next;
      $list =~ s/, (Co-)?Producers?//g;
      @people = split /, | and | & |;\s+/, $list;
      s/.* by // for @people;
    }
    $movie{$what}{$for} = $oscars{$year}{$for} = \@people;
  }

  return (\%oscars, \%movie);
}

1;
