package OscarDance;
use Dancer ':syntax';

our $VERSION = '0.1';

use Dancer::Plugin::SimpleCRUD;
use Dancer::Plugin::Database;
use TMDB;
use URI;

get '/' => sub {
    template 'index';
};

post '/search' => sub {
  my $query = param( 'query' );
  my @results = database->quick_select( creative_work => { title => { like => "%$query%" } }, { order_by => 'title' } );
  template results => { query => $query, results => \@results };
};

get '/show/:id' => \&show_id;

sub show_id
{
  my $id = param( 'id' );
  my $a_ref = database->selectall_arrayref( <<'EOSQL', { Slice => {} }, $id );
    SELECT c.title, p.name, o.year, o.category, m.review
      FROM movie m, oscar o, winner_oscar_assoc wo, person p, actor_movie_assoc am, creative_work c
     WHERE c.id = m.creative_work_id
       AND m.id = o.for_movie_id
       AND o.id = wo.oscar_id
       AND wo.winner_person_id = p.id
       AND p.id = am.person_id
       AND c.id = ?
EOSQL
  my $poster_url = poster_url( $a_ref->[0]{title}, $a_ref->[0]{year} );
  template show => { id => $id, results => $a_ref, poster => $poster_url  };
}

sub poster_url
{
  my ($title, $year) = @_;

  my $apikey = config->{tmdb_key};
  my $tmdb = TMDB->new( apikey => $apikey );
  my $search = $tmdb->search( include_adult => 'false' );
  my ($result) = $search->movie( "$title ($year)" ) or return;
  my $movie = $tmdb->movie( id => $result->{id} );
  my $config = $tmdb->config;
  my $base_url = $config->img_base_url();
  my $poster_sizes = $config->img_poster_sizes();
  my $path = $movie->poster;         # Main poster
  my $preferred_size = $poster_sizes->[1] || $poster_sizes->[0];
  return URI->new_abs( "$preferred_size/$path", $base_url );
}


post '/review/:id' => sub {
  my $id = param( 'id' );
  my $movie_id = database->quick_lookup( movie => { creative_work_id => $id }, 'id' );
  database->quick_update( movie => { id => $movie_id }, { review => param( 'review' ) } );
  show_id();
};

simple_crud (
  prefix => '/browse_movie',
  db_table => 'movie',
  foreign_keys => {
                   creative_work_id => {
                                       table => 'creative_work',
                                       key_column => 'id',
                                       label_column => 'title' },
                  }
);

simple_crud (
  prefix => '/browse_person',
  db_table => 'person',
);

simple_crud (
  prefix => '/browse_oscar',
  db_table => 'oscar',
  foreign_keys => {
                   for_movie_id => {
                                       table => 'movie',
                                       key_column => 'id',
                                       label_column => 'creative_work_id' },
                  }
);

true;
